<?php
/**
 * VersionTracker class to create payment widget, send payment request and receive payment response from payment gateway
 *
 * @package     modules.payment
 * @subpackage  vrpayecommerce
 * @since       1.0.0
 */

include_once(dirname(__FILE__).'/core.php');

class VersionTracker
{
    private static $version_tracker_url = 'http://api.dbserver.payreto.eu/v1/tracker';

    /**
     * Get version tracker URL
     *
     * @return  string
     */
    private static function getVersionTrackerUrl()
    {
        return self::$version_tracker_url;
    }

    /**
     * Get version tracker parameter
     *
     * @param   array  $version_data
     * @return  string
     */
    private static function getVersionTrackerParameter($version_data)
    {
        $data = 'transaction_mode=' .$version_data['transaction_mode'].
                '&ip_address=' .$version_data['ip_address'].
                '&shop_version=' .$version_data['shop_version'].
                '&plugin_version=' .$version_data['plugin_version'].
                '&client=' .$version_data['client'].
                '&hash=' .md5($version_data['shop_version'].$version_data['plugin_version'].$version_data['client']);

        if ($version_data['shop_system']) {
            $data .= '&shop_system=' .$version_data['shop_system'];
        }
        if ($version_data['email']) {
            $data .= '&email=' .$version_data['email'];
        }
        if ($version_data['merchant_id']) {
            $data .= '&merchant_id=' .$version_data['merchant_id'];
        }
        if ($version_data['shop_url']) {
            $data .= '&shop_url=' .$version_data['shop_url'];
        }
        if ($version_data['merchant_location']) {
            $data .= '&merchant_location=' .$version_data['merchant_location'];
        }

        return $data;
    }

    /**
     * Send version tracker to version tracker gateway
     *
     * @param   array  $version_data
     * @return  string
     */
    public static function sendVersionTracker($version_data)
    {
        $post_data = self::getVersionTrackerParameter($version_data);
        $url = self::getVersionTrackerUrl();
        return VRpayecommercePaymentCore::getResponseData($post_data, $url, $version_data['transaction_mode']);
    }
}
